# frozen_string_literal: true

class AuthenticatableController < Authenticatable.config.parent_controller.constantize
  helper_method :authenticatable_scope, :resource, :resource_name, :resource_class,
                :identifier_column, :identifier_label

  # Make @resource available in views without @ prefix.
  # Example:
  #   resource => @resource
  attr_reader :resource

  # Convinience method to access request.env["authenticatable"]
  # Example:
  #   authenticatable.sign_in() => request.env["authenticatable"].sign_in()
  def authenticatable
    @authenticatable ||= request.env["authenticatable"]
  end

  # Attempt to find the scope for authenticatable based
  # on env variable set by router constraint.
  def authenticatable_scope
    @authenticatable_scope ||= request.env["authenticatable.scope"]
  end

  # Returns the resource_name from current scope.
  # Example:
  #   resource_name => user
  def resource_name
    authenticatable_scope.singular_name
  end

  # Returns the resource_name from current scope.
  # Example:
  #   resource_class => User
  def resource_class
    authenticatable_scope.klass
  end

  # Returns the choosen identifier column for the
  # resource in the current scope.
  # Example:
  #   identifier_column => :email
  delegate :identifier_column, to: :resource_class

  # Returns the choosen identifier column as string.
  # Example:
  #   identifier_label => Email
  def identifier_label
    identifier_column.to_s.titleize
  end

  # Redirect to after_signed_in_path if authenticated
  def require_unauthenticated!
    redirect_to after_sign_in_path(resource, resource_name) if authenticatable.authenticate(resource_name)
  end

  protected

  # Sets the flash message with :key, using I18n. By default you are able
  # to set up your messages using specific resource scope, and if no message is
  # found we look to the default scope. Set the "now" options key to a true
  # value to populate the flash.now hash in lieu of the default flash hash (so
  # the flash message will be available to the current action instead of the
  # next action).
  def set_flash_message(key, kind, options = {})
    return unless flashing_format?

    message = find_message(kind, options)
    return if message.blank?

    if options[:now]
      flash.now[key] = message
    else
      flash[key] = message
    end
  end

  # Find message in I18n translation files.
  def find_message(kind, options = {})
    options[:scope] ||= translation_scope
    I18n.t(kind, **options)
  end

  # Default translation scope for messages
  def translation_scope
    "authenticatable.#{controller_name}"
  end

  # Check if flash messages should be emitted. Default is to do it on
  # navigational formats
  def flashing_format?
    request.respond_to?(:flash) && navigational_format?
  end

  # Check if current request format is navigational. Formats like
  # :html & :turbo_stream should redirect and show flash messages,
  # but formats like :xml or :json, should return 401
  def navigational_format?
    Authenticatable.config.navigational_formats.include?(request.format.try(:ref))
  end
end
