# frozen_string_literal: true

require "authenticatable/controllers"
require "authenticatable/errors"
require "authenticatable/manager"
require "authenticatable/models"
require "authenticatable/rails/routes"
require "authenticatable/scope"
require "authenticatable/proxy"
require "authenticatable/serializers"
require "authenticatable/token"
require "authenticatable/version"
require "authenticatable/engine"

require "dry-configurable"

# Authenticatable is an authentication solution for Rails. All related modules,
# classes and methods will be namespaced under the namespace 'Authenticatable':
module Authenticatable
  extend Dry::Configurable

  # Store authenticatable resources as scopes.
  @scopes = {}

  # Default extensions to load into your authenticatable models. This can for example
  # be a custom Warden strategy, concerns with validations, or basically anything that
  # modifies the default behavior of your authenticatable model. Feel free to develop
  # add publish your own extensions online. We'd be happy to see what you can accomplish.
  #
  #   email_validator: Validates emails with the valid_email2 gem. You can remove this
  #                    if you want to implement your own email validations or if you
  #                    don't want to validate emails at all.
  #
  #   password:        Enables password authentication with has_secure_password. This extension
  #                    includes a Warden Strategy for authentication with password. It does also
  #                    add password validations to your authenticatable model.
  #
  setting :default_extensions, %i[identifier password email_validator]

  # Default column to use when looking up an authenticatable record in the database.
  # Can be for example email or a username. Default is :email. This can also be changed
  # for an individual resource/scope by passing :identifier to the authentication
  # method in your model:
  #
  #   authenticatable identifier: :username
  #
  setting :default_identifier, :email

  # Set the default user scope
  setting :default_scope, :user

  # The controller class that all Authenticatable controllers will inherit from.
  # Defaults to `ApplicationController`.
  setting :parent_controller, "ApplicationController"

  # The mailer class that all Authenticatable mailers will inherit from.
  # Defaults to `ApplicationMailer`.
  setting :parent_mailer, "ApplicationMailer"

  # Range validation for password length
  setting :password_length, 6..128

  # Requests method(s) allowed for sign out. You can for example add :get
  # to enable sign out by visiting /users/sign_out.
  setting :allowed_sign_out_methods, %i[delete]

  # Which formats should be treated as navigational. Navigational formats
  # like :html & :turbo_stream should redirect and show flash messages,
  # but other formats like :xml or :json, should return 401. The "*/*"
  # format below is required to match Internet Explorer.
  setting :navigational_formats, ["*/*", :html, :turbo_stream]

  # Email regex used to validate email formats. It simply asserts that
  # one (and only one) @ exists in the given string. This is mainly
  # to give user feedback and not to assert the e-mail validity.
  setting :email_regexp, /\A[^@\s]+@[^@\s]+\z/

  # Specify for how long a reset_password_token will be valid.
  # When the time expires, their token will be invalid and they
  # have to request a "reset password link" to recover their account.
  setting :reset_password_expiration_time, 1.hour

  class << self
    attr_reader :scopes

    def reset_default_values!
      remove_instance_variable :@config
    end

    # Add a authenticatable model to list of scopes. It could be for example
    # :users, :admins, :subscribers or any other model that should be authenticatable.
    def add_scope(resource, options = {})
      scope = Authenticatable::Scope.new(resource, options)
      @scopes[scope.singular_name.to_sym] = scope

      # Define helpers like current_{scope}, authenticate_{scope}!
      # {scope}_signed_in? etc for the given scope that can be accessed from all controllers.
      Authenticatable::Controllers::Helpers.define_helpers(scope.singular_name)

      scope
    end
  end
end
