# frozen_string_literal: true

require "active_support/concern"

module Authenticatable
  module Models
    module Identifier
      extend ActiveSupport::Concern

      # Since the identifier_column can be set dynamically (to for example email or username)
      # we'll need a common attribute to access the authenticated resource identifier.
      def identifier
        self[self.class.identifier_column]
      end

      class_methods do
        # Search for a resource by the choosen identifier.
        # find_by_identifier('foo@bar.com') is equivalent to find_by(email: 'foo@bar.com')
        def find_by_identifier(value)
          find_by("#{identifier_column}": normalize_identifier(value))
        end

        # Ensure identifier is a downcase string without spaces.
        # Example:
        #   fOo@b aR.com => foo@bar.com
        def normalize_identifier(identifier)
          identifier.to_s.downcase.gsub(/\s+/, "")
        end

        # Identifying column to use when looking up an authenticatable record in the database.
        # Can be for example email or a username. Default is email.
        def identifier_column
          @identifier_column || Authenticatable.config.default_identifier
        end

        # Convient method to update the identifier_column.
        def identify_by(column)
          @identifier_column = column
        end
      end
    end
  end
end
