# frozen_string_literal: true

require "active_support/concern"
require "bcrypt"

module Authenticatable
  module Models
    module Password
      extend ActiveSupport::Concern

      included do
        has_secure_password
        validates_presence_of :password_confirmation, if: :password_digest_changed?
        validates :password, length: Authenticatable.config.password_length, if: :password_digest_changed?
      end

      class_methods do
        def authenticate_with_identifier(identifier, password)
          if (resource = find_by_identifier(identifier))
            resource&.authenticate(password) ? resource : nil
          else
            prevent_timing_attack
          end
        end

        def find_by_reset_password_token(token)
          return nil if token.blank?

          expiration_time = Authenticatable.config.reset_password_expiration_time
          where(reset_password_token: token).where("reset_password_sent_at > ?", expiration_time.ago).first
        end

        # Hash a random password even when a resource doesn't exist for the given identifier.
        # This is necessary to protect against timing/enumeration attacks - e.g. the request
        # is faster when a resource doesn't exist in the database if the password hashing
        # algorithm is not called.
        def prevent_timing_attack
          new(password: "*")
          nil
        end
      end

      # Sets the :reset_password_token attribute to a random token
      # generated by Authenticatable::Token
      def flush_reset_password_token
        self.reset_password_token = Authenticatable::Token.new
        self.reset_password_sent_at = Time.current
      end

      # Sets the :reset_password_token attribute to a random token
      # generated by Authenticatable::Token and saves the record
      # without running validations.
      def reset_password_token!
        flush_reset_password_token
        save(validate: false)
      end

      # Validates and sets the new password from secure_params and
      # sets reset_password_token to nil.
      def update_password(secure_params)
        if secure_params[:password].blank?
          errors.add(:password, :blank)
          return false
        end

        self.password = secure_params[:password]
        self.password_confirmation = secure_params[:password_confirmation]
        flush_reset_password_token if valid?
        save
      end
    end
  end
end
