# frozen_string_literal: true

module Authenticatable
  # The authenticatable session, persisted in env["authenticatable"]
  class Proxy
    def initialize(env)
      @env = env
      @current = {}
      @config = Authenticatable.config
    end

    # Check if the given scope is already signed in or else run authenticated strategies it.
    # This does not halt the flow of control and is a passive attempt to authenticate only.
    # :api: public
    def authenticate(scope)
      current?(scope) || run_serializers_for(scope) || nil
    end

    # Same as +authenticate+ except on failure it will trow a
    # :unauthenticated symbol.
    # :api: public
    def authenticate!(scope)
      unless (resource = authenticate(scope))
        raise Authenticatable::UnauthenticatedError
      end

      resource
    end

    # Add a resource/scope into the @current hash. This method is called after an authentication
    # strategy has succeeded, but can also be used in tests/debugging to manually sign in a user.
    #
    # PLEASE NOTICE that this method DOES NOT perform any authentication strategies.
    # To authenticate a user, you must use the +authenticate+ method instead.
    def sign_in(resource, serializer = nil)
      scope = scope_from_resource(resource)
      @current[scope] = resource

      unless serializer.nil?
        klass = initialize_serializer(scope, serializer)
        klass.store(resource.id)
      end

      resource
    end

    # Remove a resource/scope from the @current hash
    def sign_out(resource, serializer = nil)
      scope = scope_from_resource(resource)
      @current[scope] = nil

      unless serializer.nil?
        klass = initialize_serializer(scope, serializer)
        klass.purge!
      end

      true
    end

    # Provides access to the user object in a given scope for a request.
    # Will be nil if not logged in or an instance of the resource if logged in.
    #
    # Examples:
    #   env['authenticatable'].current?(:user) => #<User>
    #   env['authenticatable'].current?(:admin) => #<Admin>
    def current?(scope)
      scope = scope.to_sym # Make sure to always use symbols.
      @current[scope]
    end

    private

    # :api: private
    def run_serializers_for(scope)
      serializers = %i[session]
      serializers.each do |name|
        serializer = initialize_serializer(scope, name)
        if (record = serializer.fetch)
          sign_in(record)
          return record
        end
      end
      nil
    end

    # Return a constantized class by scope name.
    def initialize_serializer(scope, name)
      class_name = name.to_s.classify
      klass = "Authenticatable::Serializers::#{class_name}".constantize
      klass.new(@env, scope)
    end

    # Convert an instance to a scope symbol by
    # returning the param_key from ActiveModel#model_name
    # Examples:
    #   scope_from_resource(#<User>) => :user
    #   scope_from_resource(#<Admin>) => :admin
    def scope_from_resource(resource)
      scope = resource.model_name.param_key
      scope.to_sym # Make sure to always use symbols.
    end
  end
end
