# frozen_string_literal: true

module ActionDispatch
  module Routing
    class Mapper
      def authenticatable(resource, options = {})
        # Placeholder method for authenticatable generator.
        scope = Authenticatable.add_scope(resource, options)
        authenticatable_scope scope do
          authenticatable_sessions(scope)
          authenticatable_registrations(scope)
          authenticatable_passwords(scope)
        end
      end

      protected

      def authenticatable_scope(scope, &block)
        constraint = lambda do |request|
          request.env["authenticatable.scope"] = scope
          true
        end

        constraints(constraint) do
          scope scope.path, as: scope.singular_name, &block
        end
      end

      def authenticatable_sessions(scope)
        return if scope.skip.include? :sessions

        resource :session, only: [], path: "", controller: scope.controllers[:sessions] do
          get :new, path: scope.path_names[:sign_in], as: :new
          post :create, path: scope.path_names[:sign_in]
          match :destroy, path: scope.path_names[:sign_out], as: :destroy,
                          via: Authenticatable.config.allowed_sign_out_methods
        end
      end

      def authenticatable_registrations(scope)
        return if scope.skip.include? :registrations

        resource :registration, only: [], path: "", controller: scope.controllers[:registrations] do
          get :new, path: scope.path_names[:sign_up], as: :new
          post :create, path: scope.path_names[:sign_up]
        end
      end

      def authenticatable_passwords(scope)
        return if scope.skip.include? :passwords

        resource :password, only: [], path: "", controller: scope.controllers[:passwords] do
          get :new, path: scope.path_names[:forgot_password], as: :new
          post :create, path: scope.path_names[:forgot_password]
          get :edit, path: scope.path_names[:reset_password], as: :edit
          patch :update, path: scope.path_names[:reset_password], as: :update
        end
      end
    end
  end
end
