# frozen_string_literal: true

require "active_support/core_ext/string/inflections"

# rubocop:disable Layout/EmptyLinesAroundAttributeAccessor
module Authenticatable
  class Scope
    # The resource name added by the authenticatable-route (for example :users)
    attr_accessor :name
    @name = nil

    # The resource name converted to singluar (used by for example url helpers)
    #   authenticatable :users generates helper methods like:
    #   new_user_session_path and user_signed_in?
    attr_accessor :singular_name
    @singular_name = nil

    # The resource name converted to plural (used to generate controllers etc). This is used
    # as a fallback in case the user defines a resource in plural like:
    #   authenticatable :user
    attr_accessor :plural_name
    @plural_name = nil

    # You can customize a resource path to something other than the default,
    # including the possibility to change path for I18n:
    #    authenticatable :users, { path: I18n.translate('routes.account') }
    attr_accessor :path
    @path = {}

    # You can customize a resource's path_names to something other than the default,
    # including the possibility to change path names for I18n:
    #    authenticatable :users, { path_names: { sign_in: I18n.translate('routes.login') } }
    attr_accessor :path_names
    @path_names = {}

    # You can also specify another controller than the default one in case you
    # want to override some controller actions.
    attr_accessor :controllers
    @controllers = {}

    # You can also specify controllers you want to skip. For example if you
    # want to disable registrations for an Admin resource.
    attr_accessor :skip
    @skip = {}

    # Create a new Scope resource that can be added to Authenticatable.scopes
    def initialize(resource_name, options = {})
      @name = resource_name.to_s
      @singular_name = ActiveSupport::Inflector.singularize(resource_name)
      @plural_name = ActiveSupport::Inflector.pluralize(resource_name)
      @path = options[:path] || default_path
      @path_names = default_path_names.merge(options[:path_names] || {})
      @controllers = default_controllers.merge(options[:controllers] || {})
      @skip = *options[:skip]
      @skip = @skip.map(&:to_sym)
    end

    def classify
      ActiveSupport::Inflector.classify @singular_name
    end

    def klass
      classify.constantize
    end

    # Create magic predicates for verifying that a
    # controller exists and hasn't been skipped for
    # the current scope in routes.rb.
    #
    # Example
    #   current_scope.registrations? => false
    # when registrations are skipped in routes:
    #   authenticatable :admins, skip: [:registrations]
    #
    # rubocop:disable Style/MissingRespondToMissing
    def method_missing(method_name)
      @controllers.each do |controller, _routes|
        return @skip.exclude?(controller) if method_name.to_s == "#{controller}?"
      end

      super # return NoMethodError
    end
    # rubocop:enable Style/MissingRespondToMissing

    private

    def default_path
      @plural_name
    end

    def default_path_names
      {
        sign_in: "sign_in",
        sign_up: "sign_up",
        sign_out: "sign_out",
        forgot_password: "forgot_password",
        reset_password: "reset_password"
      }
    end

    def default_controllers
      {
        sessions: "authenticatable/sessions",
        registrations: "authenticatable/registrations",
        passwords: "authenticatable/passwords"
      }
    end
  end
end
# rubocop:enable Layout/EmptyLinesAroundAttributeAccessor
