# frozen_string_literal: true

module Authenticatable
  module Serializers
    class Session < Authenticatable::Serializers::Base
      # Fetch record from Rack session.
      # Example:
      #   serializer.fetch(:user) => <#User>
      def fetch
        return nil unless (record_id = request.session[session_key])

        resource_class.find_by(id: record_id)
      end

      # Store record id in Rack session.
      # Usage:
      #   serializer.store(@resource)
      def store(id)
        delete_csrf_token
        renew_session_id
        request.session[session_key] = id
      end

      # Delete record id from Rack session
      # Usage:
      #   serializer.purge
      def purge!
        request.session.delete(session_key)
      end

      private

      # Protection against sessions fixation attacks by clearing the session_id on authentication.
      def renew_session_id
        return if request.env["rack.session.options"].blank?

        request.env["rack.session.options"][:renew] = true
      end

      # Protect against cross-site request forgery (CSRF) by cleaning up the CSRF Token on authentication.
      def delete_csrf_token
        request.session.delete("_csrf_token")
      end

      # Returns the session key for a scoped authenticatable session.
      def session_key
        :"authenticatable_#{@scope}_id"
      end
    end
  end
end
